@@ -18,6 +18,7 @@
  */
 #ifndef _GIAC_GLOBAL_H
 #define _GIAC_GLOBAL_H
+//#define USE_OBJET_BIDON
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
@@ -34,7 +35,7 @@
 #if defined VISUALC || defined BESTA_OS || defined FREERTOS
 typedef long pid_t;
 #else // VISUALC
-#if !defined(__MINGW_H) && !defined(NSPIRE) && !defined(FXCG) && !defined(__ANDROID__) && !defined(NSPIRE_NEWLIB) && !defined(OSX) && !defined(IOS) && !defined(OSXIOS) && !defined(FIR_LINUX) && !defined(PRIMEWEBASM)
+#if !defined(__MINGW_H) && !defined(HP39) && !defined(NSPIRE) && !defined(FXCG) && !defined(__ANDROID__) && !defined(NSPIRE_NEWLIB) && !defined(OSX) && !defined(IOS) && !defined(OSXIOS) && !defined(FIR_LINUX) && !defined(PRIMEWEBASM)
 #include "wince_replacements.h"
 #endif
 #ifdef __MINGW_H
@@ -113,8 +114,10 @@ bool dfu_get_scriptstore_addr(size_t & start,size_t & taille);
 bool dfu_get_scriptstore(const char * fname);
 bool dfu_send_scriptstore(const char * fname);
 bool dfu_send_rescue(const char * fname);
+bool dfu_send_bootloader(const char * fname);
 const int nwstoresize1=0x8000,nwstoresize2=0x8014;
-bool dfu_send_firmware(const char * fname);
+// send to 0x90000000+offset*0x10000
+bool dfu_send_firmware(const char * fname,int offset);
 bool dfu_send_apps(const char * fname);
 bool dfu_update_khicas(const char * fname); 
 
@@ -164,6 +167,8 @@ int tar_filebrowser(const char * buf,const char ** filenames,int maxrecords,cons
 int flash_synchronize(const char * buffer,const std::vector<fileinfo_t> & finfo,size_t * tar_first_modif_offsetptr);
 ulonglong fromstring8(const char * ptr);
 std::string toString8(longlong chksum);
+void WriteMemory(char * target,const char * src,size_t length);
+void erase_sector(const char * buf);
 
 // empty trash: files marked as non readable are really removed
 // this will do 1 sector write from first sector where a file is marked to be removed to the end 
@@ -184,6 +189,37 @@ char * numworks_gettar(size_t & tar_first_modif_offset);
 bool numworks_sendtar(char * buffer,size_t buffersize,size_t tar_first_modif_offset=0);
 #endif
 
+//sha256 support
+#if !defined USE_GMP_REPLACEMENTS && !defined GIAC_HAS_STO_38
+#ifdef __cplusplus
+extern "C" {
+#endif
+/*************************** HEADER FILES ***************************/
+#include <stddef.h>
+
+/****************************** MACROS ******************************/
+#define SHA256_BLOCK_SIZE 32            // SHA256 outputs a 32 byte digest
+
+/**************************** DATA TYPES ****************************/
+typedef unsigned char BYTE;             // 8-bit byte
+typedef unsigned int  WORD32;             // 32-bit word, change to "long" for 16-bit machines
+
+typedef struct {
+	BYTE data[64];
+	WORD32 datalen;
+	unsigned long long bitlen;
+	WORD32 state[8];
+} SHA256_CTX;
+
+/*********************** FUNCTION DECLARATIONS **********************/
+void giac_sha256_init(SHA256_CTX *ctx);
+void giac_sha256_update(SHA256_CTX *ctx, const BYTE data[], size_t len);
+void giac_sha256_final(SHA256_CTX *ctx, BYTE hash[]);
+#ifdef __cplusplus
+}
+#endif
+#endif // sha256  
+
 #ifndef NO_NAMESPACE_GIAC
 namespace giac {
 #endif // ndef NO_NAMESPACE_GIAC
@@ -333,7 +369,10 @@ Boolean isLegalUTF8Sequence(const UTF8 *source, const UTF8 *sourceEnd);
   extern int FACTORIAL_SIZE_LIMIT;
   extern int GAMMA_LIMIT;
   extern int LIST_SIZE_LIMIT;
+  extern int ABERTH_NMAX; // max number of iterations
+  extern int ABERTH_NBITSMAX; // max precision
   extern int NEWTON_DEFAULT_ITERATION;
+  extern int NEWTON_MAX_RANDOM_RESTART;
   extern int DEFAULT_EVAL_LEVEL;
   extern int PARENTHESIS_NWAIT;
   extern int MAX_PROD_EXPAND_SIZE;
@@ -369,6 +408,7 @@ Boolean isLegalUTF8Sequence(const UTF8 *source, const UTF8 *sourceEnd);
   extern double gbasis_reinject_ratio;
   extern double gbasis_reinject_speed_ratio;
   extern int gbasis_logz_age_sort,gbasis_stop,rur_do_gbasis,rur_do_certify,rur_certify_maxthreads;
+  extern bool rur_error_ifnot0dimensional; // error code or compute gbasis
 
   extern int PROOT_FACTOR_MAXDEG;
   extern int ABS_NBITS_EVALF;
@@ -547,6 +587,7 @@ throw(std::runtime_error("Stopped by user interruption.")); \
     size_t stacksize;
     void * stackaddr;
 #endif
+    size_t stack;
     thread_param();
   };
 
@@ -618,6 +659,9 @@ throw(std::runtime_error("Stopped by user interruption.")); \
     // bit 0-8=radius, bit9-17 angle1, bit 18-26 angle2, bit 27=1 filled  or 0 
     // <0 fill a polygon from previous turtle positions
     logo_turtle(): x(100),y(100),theta(0),visible(true),mark(true),direct(true),color(0),turtle_width(1),radius(0) {}
+    inline bool equal_except_nomark(const logo_turtle &t) const {
+      return x==t.x && y==t.y && turtle_width==t.turtle_width && s==t.s && radius==t.radius;
+    }
   };
 #else // KHICAS
   struct logo_turtle {
@@ -633,6 +677,9 @@ throw(std::runtime_error("Stopped by user interruption.")); \
     // <0 fill a polygon from previous turtle positions
     std::string s;
     void * widget;
+    inline bool equal_except_nomark(const logo_turtle &t) const {
+      return x==t.x && y==t.y  && turtle_width==t.turtle_width && s==t.s &&  radius==t.radius;
+    }
 #ifdef IPAQ
     logo_turtle(): x(70),y(70),theta(0),visible(true),mark(true),direct(true),color(0),turtle_width(1),radius(0),widget(0) {}
 #else
@@ -772,6 +819,7 @@ throw(std::runtime_error("Stopped by user interruption.")); \
 
   context * clone_context(const context *);
   void init_context(context * ptr);
+  void clear_context(context * ptr);
 
   extern const context * context0;
   std::vector<context *> & context_list();
